package org.w3c.rdfpic.core;
import java.io.*;
import java.net.*;
import org.w3c.www.protocol.http.*;
import org.w3c.www.mime.*;
/**
 * ProtocolHandler abstracts read/write access to a URL independant of the protocol.  When initialized with
 * its url, a ProtocolHandler instance buffers that entire URL into memory.  ProtocolHandler's main advantage
 * is HTTP 1.1 PUT support, allowing write access over the web.  
 */

public class ProtocolHandler
{
    URL url;
    SubPHandler sph;
    byte[] readBuf;
    /**
     * Creates a new ProtocolHandler instance with the given url and buffers it's contents
     */
    public ProtocolHandler(URL url) throws PHException
    {
	this.url=url;
	readBuf=null;
	sph=null;
	if (url==null) throw new PH_noURL("No URL specified");
	String protocol = url.getProtocol();
	if (protocol.equals("file")) filePH();
	else if(protocol.equals("http")) httpPH();
	else throw new PH_unknownProtocol("Protocol "+ protocol+" unknown");
    }

    /**
     * Initilizes a helper class to deal with http:// urls.  (Called by constructor)
     */
    void httpPH()throws PHException
    {
	sph = new HttpPHandler(url);
	readBuf=sph.getReadBuf();
    }
    /**
     * Initializes a helper class to deal with file:// urls. (Called by constructor)
     */
    void filePH() throws PHException
    {
	sph = new FilePHandler(url);
	readBuf=sph.getReadBuf();
    }
    /**
     * Returns a readable InputStream from the url.
     */
    public InputStream getIS() throws PH_noRead
    {
	if(readBuf==null) throw new PH_noRead();
	return new ByteArrayInputStream(readBuf);
    }
    /**
     * Returns the size of the file loaded (measured in bytes)
     */
    public int fileSize() throws PH_noRead
    {
	if(readBuf==null||!sph.canRead()) throw new PH_noRead();
	return readBuf.length;
    }

    /**
     * Reads the entire file.
     */
    public byte[] read() throws PH_noRead
    {
	if(readBuf==null||!sph.canRead()) throw new PH_noRead();
	byte[] retval = new byte[readBuf.length];
	System.arraycopy(readBuf,0,retval,0,readBuf.length);
	return retval;
    }
    /**
     * Writes the entire file (overwriting if necessary).
     */
    public void write(byte[] data) throws PHException
    {
	if(!sph.canWrite()) throw new PH_noWrite();
	sph.write(data);
	readBuf=new byte[data.length];
	System.arraycopy(data,0,readBuf,0,data.length);
    }
    /**
     * Returns, if available, the MIME type of the url's contents.  (not fully implemented yet).
     */
    public String contentType() throws PHException
    {
	return sph.contentType();
    }
    /**
     * Returns whether the file is readable. (The return value is guaranteed accurate.)
     */
    public boolean canRead()
    {
	return sph.canRead();
    }
    /**
     * Returns whether the file is writable.  (May return true even when not writable.)
     */
    public boolean canWrite()
    {
	return sph.canWrite();
    }
}
/**
 * A class exporting this interface must deal with a specific protocol.
 * ProtocolHandler passes the appropriate commands on when necessary.
 */
interface SubPHandler
{
    public boolean canRead();
    public boolean canWrite();
    /**
     * After initialization, all data must be buffered.  This allows ProtocolHandler to
     * take care of the buffering from now on.
     */
    byte[] getReadBuf();
    /**
     * Overwrites file with given data...
     */
    void write(byte[] data) throws PHException;
    /**
     * Returns the MIME type of the loaded file.
     */
    String contentType() throws PHException;
}

/**
 * FilePHandler is an implementation of SubPHandler for the file:// protocol.
 */
class FilePHandler implements SubPHandler
{
    byte[] readBuf;
    File theFile;
    public boolean canRead(){return readable;}
    public boolean canWrite(){return writable;}
    boolean writable,readable;
    public FilePHandler(URL url) throws PHException
    {
	writable=readable=false;
	try {theFile = new File(url.getFile());}
	catch(NullPointerException npe) {throw new PH_badURL("HUH????");}
	if(!theFile.exists()) throw new PH_notFound("File not found");
	try {
	    if(!theFile.canRead()) throw new PH_noRead("no read-access to: "+theFile.toString());
	} catch(SecurityException se) {throw new PHException("Unable to check readablility");}
	FileInputStream fis=null;
	try {
	    fis = new FileInputStream(theFile);
	} catch (FileNotFoundException fnfe) {throw new PHException("File dissapeared?");}
	catch (SecurityException se) {throw new PHException("Unexpectedly unable to read file (" +se.toString()+").");}
	try {
	    readBuf = new byte[(int)theFile.length()];
	} catch (SecurityException se) {throw new PHException("Unexpectedly unable to read file (" +se.toString()+").");}
	try {
	    fis.read(readBuf);
	} catch(IOException ioe) {throw new PH_IOError(ioe.toString());}
	try {
	    if(fis.read()!=-1) throw new PH_IOError("File length inaccurate");
	} catch(IOException ioe) {throw new PH_IOError("Problem at EOF");}
	readable=true;
	try {
	    if(!theFile.canWrite()) return;//throw new PH_noWrite("no write access to: "+theFile.toString());
	} catch (SecurityException se) {throw new PHWarning("Unable to check writability");}
	writable=true;
	}

    public byte[] getReadBuf() {return readBuf;}

    public void write(byte[] data) throws PHException
    {
	if(!writable) throw new PH_noWrite();
	FileOutputStream fos=null;
	try {
	    fos = new FileOutputStream(theFile);
	} catch (FileNotFoundException fnfe) {throw new PHException("Unable to open output stream");}
	catch (SecurityException se) {throw new PHException("Not allowed to write");}
	catch (IOException ioe) {throw new PHException("weird (undocumented) exception: "+ioe.toString());}

	try {
	    fos.write(data);
	} catch (IOException ioe) {throw new PH_IOError("Unexpected write error: "+ioe.toString());}
	try {
	fos.close();
	} catch (IOException ioe) {throw new PH_IOError("Unable to close file: "+ioe.toString());}
    }

    public String contentType()
    {
	return "image/jpeg";//:-)
    }
}

/**
 * HttpPHandler is an implementation of SubPHandler that deals with the HTTP 1.1 protocol.
 */

    class HttpPHandler implements SubPHandler  
{
    URL url;
    byte[] readBuf;
    HttpManager manager;
    public boolean canRead(){return readable;}
    public boolean canWrite(){return writable;}
    boolean readable, writable;
    public HttpPHandler(URL url) throws PHException
    {
	readable=writable=false;
	this.url=url;
	// Get the manager, and define some global headers:
	manager = HttpManager.getManager();
	manager.setGlobalHeader("User-Agent", "RdfPic/2.1");
	manager.setGlobalHeader("Accept", "*/*;q=1.0");
	manager.setGlobalHeader("Accept-Encoding", "gzip");
	//do initial request
	Request request = manager.createRequest();
	request.setURL(url);
	request.setMethod("GET");
	Reply reply=null;
	try {
	    reply   = manager.runRequest(request);
	} catch(HttpException he) {
	    throw new PHException("Unable to run request:\n("+he.toString()+")");
	}
	InputStream in;
	try {
	    in = reply.getInputStream();
	} catch(IOException ioe) {
	    throw new PH_noRead("Input Stream not in reply");
	}
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	byte buf[] = new byte[4096];
	int  cnt   = 0;
	try {
	    while ((cnt = in.read(buf)) > 0) 
		baos.write(buf,0,cnt);
	} catch (IOException ioe) {
	    throw new PH_IOError("While reading from http InputStream: "+ioe.getMessage());
	}
	readBuf= baos.toByteArray();
	try {
	    in.close();
	} catch(IOException ioe) {
	    throw new PH_IOError("While closing http InputStream: "+ioe.getMessage());
	}
	manager.sync();
	readable=writable=true;
    }
    public byte[] getReadBuf() {return readBuf;}
    public void write(byte[] data) throws PHException
    {
	Request request = manager.createRequest();
	request.setURL(url);
	request.setMethod("PUT");
	try {
	    request.setContentType(new org.w3c.www.mime.MimeType("image/jpeg"));
	} catch (MimeTypeFormatException mtfe) {
	    throw new Error("Mimetype image/jpeg error: "+mtfe.toString());
	}
	request.setContentLength(data.length);
	request.setOutputStream(new ByteArrayInputStream(data));
	try {
	    manager.runRequest(request);
	} catch (HttpException he) {
	    throw new PH_noWrite(he.toString());
	}
	
	manager.sync();
    }
    public String contentType()
    {
	return "image/jpeg";
    }
}
