package org.w3c.rdfpic.ui;
import org.w3c.rdfpic.core.*;
import javax.swing.*;
import java.awt.*;
import java.io.IOException;
import java.awt.event.*;
import java.util.*;

/**
 * Displays a tabbed pane for editing values of fields for an image.
 * Each Tab corresponds to a schema.  Within that tab, there are a number of rows.
 * Each row corresponds to a field of that schema.  To the left will be the name of
 * the field, to the right an editable MemComboBox for entering values.  Between the name and
 * field is a checkbox - if it is checked, the value is persistent.  That means that when a
 * new file is loaded, the previous value remains selected.  This is useful, as a large
 * number of photo's might be made by the same photographer in the same location with
 * the same camera.  Regardless of whether or not the values are persistent, all
 * old values can be recalled as they are stored in the MemComboBox. 
 * <p>
 * This class is an RdfDataProvider.
 */

public class SchemaTabbedPane extends JTabbedPane implements RdfDataProvider,SchemaListener,ImgListener
{
    static Font font = new Font("sans-serif", Font.PLAIN, 12);
    JComboBox[][] dialog; //dialog[a][b] means of scheme a, prop b
    JCheckBox[][] persistent;
    JScrollPane[] tabs;
    RdfPicCore core;
    TreeMap builtinMap;
    public void imgVUpdate() {}
    public SchemaTabbedPane(RdfPicCore core) 
    {
	super();
	this.core=core;
	core.addSchemaListener(this);
	core.setRdfDataProvider(this);
	core.addImgListener(this);
	builtinMap=new TreeMap();
	make();
	try {
	    core.loadData();
	} catch (RdfPicError err) {}//ohoh!!  :-)
        catch (IOException ioe){lockInput();}
    }

    /**
     * Adds a CboxMaker to use when when a specified predicate is encountered as builtin.
     */
    public void addBuiltin(String predName,CboxMaker cboxMaker) {
	builtinMap.put(predName,cboxMaker);
	remake();
    }
    
    public void removeBuiltin(String predName) {
	builtinMap.remove(predName);
	remake();
    }

    public String valOf(int schemaIndex, int predicateIndex)
    {
	return (String)dialog[schemaIndex][predicateIndex].getSelectedItem();
    }

    public void setVal(int schemaIndex, int predicateIndex,String val)
    {
	dialog[schemaIndex][predicateIndex].setSelectedItem(val);
    }

    public void remake()
    {
	removeAll();
	make();
    }
    private void make()
    {
	int size=core.numSchemas();
	dialog = new JComboBox[size][];
	persistent = new JCheckBox[size][];
	tabs   = new JScrollPane[size];
	for (int i=0; i < size; i++)
	    {
		createPane(i);
		add(tabs[i]);
	    }
    }

    // Creates a JPanel to edit the specified schemaDef object
    private void createPane(int schemaIndex) 
    {
	int numP=core.numPredicates(schemaIndex);
	JPanel panel = new JPanel(new GridBagLayout());
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets = new Insets(2, 2, 2, 2);
        constraints.weighty = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
	JComboBox cbox=null;
	dialog[schemaIndex]=new JComboBox[numP];
	persistent[schemaIndex]=new JCheckBox[numP];
	for (int i=0; i < numP; ++i) 
	    {
		Object builtinTemp;
		JCheckBox pers;
		if (i==numP-1) constraints.weighty=1.0;
		String name    = core.predicateName(schemaIndex,i);
		boolean builtin= core.isBuiltinPredicate(schemaIndex,i);
		String defaultval = core.defaultVal(schemaIndex,i);
		String tooltip = name+":\n"+core.predExpl(schemaIndex,i);
		constraints.anchor = GridBagConstraints.NORTHEAST;
		constraints.weightx = 0;
		constraints.fill = GridBagConstraints.NONE;
		constraints.gridx = 0;
		constraints.gridy = i;
                JLabel predname = new JLabel(name + " : ");
		predname.setToolTipText(tooltip);
		panel.add(predname, constraints);
		constraints.anchor = GridBagConstraints.NORTH;
		constraints.gridx = 1;
		pers = new JCheckBox();
                if(defaultval!=null) pers.setSelected(true);
		else pers.setSelected(false);
		panel.add(pers,constraints);
		persistent[schemaIndex][i]=pers;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weightx = 1.0;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridx = 2;
		if(builtin) {
		    CboxMaker cm=(CboxMaker)builtinMap.get(name);
		    if(cm==null) builtin=false;
		    else cbox=cm.newCbox(name);
		} 
		if(!builtin) {
		    cbox=new MemComboBox();
		    cbox.setEditable(true);
		    cbox.setFont(font);
		    cbox.setSelectedItem(defaultval);
		}
		dialog[schemaIndex][i]=cbox;
		panel.add(cbox,constraints);
	    }
	tabs[schemaIndex]=new JScrollPane(panel);
	//        tabs[schemaIndex].setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	tabs[schemaIndex].setName(core.schemaName(schemaIndex));
    }

    public void disablingSchema(int schemaIndex)
    {
	remove(tabs[schemaIndex]);
    }

    public void enablingSchema(int schemaIndex)
    {
	int index=0;
	for(int i=0;i<schemaIndex;i++)
	    if(core.isEnabled(i)) index++;
	add(tabs[schemaIndex],index);
    }

    public void addingSchema()
    {
	int size =core.numSchemas();//will be one more than previously
	JComboBox[][] newdialog = new JComboBox[size][];
	System.arraycopy(dialog,0,newdialog,0,size-1);
	
	JCheckBox[][] newpersistent = new JCheckBox[size][];
	System.arraycopy(persistent,0,newpersistent,0,size-1);

	JScrollPane[] newtabs = new JScrollPane[size];
	System.arraycopy(tabs, 0, newtabs, 0, size-1);
	
	dialog=newdialog;
	tabs=newtabs;
	persistent=newpersistent;

	createPane(size-1);
	add(tabs[size-1]);
	try {
	    core.loadData();
	} catch (RdfPicError rpe) {}
	catch (IOException ioe) {lockInput();}
    }
    void lockInput()
    {
	for(int i=0;i<persistent.length;++i)
	    for(int ii=0;ii<persistent[i].length;++ii)
		    dialog[i][ii].setEnabled(false);
    }

    public void isJumping(int newindex)
    {
	for(int i=0;i<persistent.length;++i)
	    for(int ii=0;ii<persistent[i].length;++ii)
		{
		    dialog[i][ii].setEnabled(false);
		    if(!persistent[i][ii].isSelected()) {
			dialog[i][ii].setSelectedItem(null);
		    }
		}
    }
    
    public void hasJumped(String errMsg)//note - different thread!!
    {
	if (errMsg!=null) {
	    lockInput();
	    return;
	}
	try {
	    core.loadData();
	} catch (RdfPicError rpe) {}
	catch (IOException ioe) {lockInput();}
	for(int i=0;i<persistent.length;++i)
	    for(int ii=0;ii<persistent[i].length;++ii)
		dialog[i][ii].setEnabled(true);
	Iterator it=builtinMap.values().iterator();
	while(it.hasNext()) {
	    try {
		CboxMaker current=(CboxMaker)it.next();
		current.newFileLoaded();
	    } catch(NoSuchElementException nsee) {}//can't happen
	}
	
    }
}

